/*
 * Copyright 2011-13 Fraunhofer ISE 
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * OpenMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 * 
 * OpenMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package org.openmuc.framework.driver.ehz;

import gnu.io.CommPortIdentifier;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;

import org.openmuc.framework.config.ArgumentSyntaxException;
import org.openmuc.framework.config.ChannelScanInformation;
import org.openmuc.framework.config.DeviceScanInformation;
import org.openmuc.framework.driver.spi.ChannelRecordContainer;
import org.openmuc.framework.driver.spi.ChannelValueContainer;
import org.openmuc.framework.driver.spi.ConnectionException;
import org.openmuc.framework.driver.spi.DeviceConnection;
import org.openmuc.framework.driver.spi.DriverService;
import org.openmuc.framework.driver.spi.RecordsReceivedListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Frederic Robra
 * 
 */
public class EhzDriver implements DriverService {

	private static Logger logger = LoggerFactory.getLogger(EhzDriver.class);

	private static final String ADDR_IEC = "iec";
	private static final String ADDR_SML = "sml";

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openmuc.framework.driver.spi.DriverService#getId()
	 */
	@Override
	public String getId() {
		return "ehz";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openmuc.framework.driver.spi.DriverService#getDescription()
	 */
	@Override
	public String getDescription() {
		return "Driver for IEC 62056-21 and SML";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openmuc.framework.driver.spi.DriverService#scanForDevices(java.lang.String, java.lang.String,
	 * java.lang.String, int)
	 */
	@Override
	public List<DeviceScanInformation> scanForDevices(String interfaceAddress, String scanFilter, String settings,
			int timeout) throws UnsupportedOperationException, ArgumentSyntaxException, IOException {
		LinkedList<DeviceScanInformation> devices = new LinkedList<DeviceScanInformation>();
		Enumeration<?> ports = CommPortIdentifier.getPortIdentifiers();
		while (ports.hasMoreElements()) {
			CommPortIdentifier port = (CommPortIdentifier) ports.nextElement();
			String serialPort = port.getName();
			logger.trace("searching for device at " + serialPort);
			URI deviceAddress = null;
			Connection connection = null;
			if (deviceAddress == null) {
				try {
					connection = new IecConnection(serialPort, 2000);
					if (connection.isWorking()) {
						logger.info("found iec device at " + serialPort);
						deviceAddress = new URI(ADDR_IEC + "://" + serialPort);
					}
					connection.close();
				} catch (Exception e) {
					logger.trace(serialPort + " is no iec device");
				}
			}
			if (deviceAddress == null) {
				try {
					connection = new SmlConnection(serialPort, 2000);
					if (connection.isWorking()) {
						logger.info("found sml device at " + serialPort);
						deviceAddress = new URI(ADDR_SML + "://" + serialPort);
					}
					connection.close();
				} catch (Exception e) {
					logger.trace(serialPort + " is no sml device");
				}
			}
			if (deviceAddress != null) {
				devices.add(new DeviceScanInformation(deviceAddress.toString(), "ehz", ""));
			}
			else {
				logger.info("no ehz device found at " + serialPort);
			}
		}

		return devices;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.openmuc.framework.driver.spi.DriverService#scanForChannels(org.openmuc.framework.driver.spi.DeviceConnection,
	 * int)
	 */
	@Override
	public List<ChannelScanInformation> scanForChannels(DeviceConnection connection, int timeout)
			throws UnsupportedOperationException, ConnectionException {
		return ((Connection) connection.getConnectionHandle()).listChannels(timeout);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openmuc.framework.driver.spi.DriverService#connect(java.lang.String, java.lang.String, java.lang.String,
	 * int)
	 */
	@Override
	public Object connect(String interfaceAddress, String deviceAddress, String settings, int timeout)
			throws ArgumentSyntaxException, ConnectionException {
		logger.trace("trying to connect to " + deviceAddress);
		try {
			URI device = new URI(deviceAddress);
			if (device.getScheme().equals(ADDR_IEC)) {
				logger.trace("connecting to iec device");
				return new IecConnection(device.getPath(), timeout);
			}
			else if (device.getScheme().equals(ADDR_SML)) {
				logger.trace("connecting to sml device");
				return new SmlConnection(device.getPath(), timeout);
			}
			throw new ConnectionException("unrecognized address scheme " + device.getScheme());
		} catch (URISyntaxException e) {
			throw new ConnectionException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openmuc.framework.driver.spi.DriverService#disconnect(org.openmuc.framework.driver.spi.DeviceConnection)
	 */
	@Override
	public void disconnect(DeviceConnection connection) {
		((Connection) connection.getConnectionHandle()).close();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openmuc.framework.driver.spi.DriverService#read(org.openmuc.framework.driver.spi.DeviceConnection,
	 * java.util.List, java.lang.Object, java.lang.String, int)
	 */
	@Override
	public Object read(DeviceConnection connection, List<ChannelRecordContainer> containers,
			Object containerListHandle, String samplingGroup, int timeout) throws UnsupportedOperationException,
			ConnectionException {
		Connection con = ((Connection) connection.getConnectionHandle());

		con.read(containers, timeout);

		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.openmuc.framework.driver.spi.DriverService#startListening(org.openmuc.framework.driver.spi.DeviceConnection,
	 * java.util.List, org.openmuc.framework.driver.spi.RecordsReceivedListener)
	 */
	@Override
	public void startListening(DeviceConnection connection, List<ChannelRecordContainer> containers,
			RecordsReceivedListener listener) throws UnsupportedOperationException, ConnectionException {
		throw new UnsupportedOperationException();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openmuc.framework.driver.spi.DriverService#write(org.openmuc.framework.driver.spi.DeviceConnection,
	 * java.util.List, java.lang.Object, int)
	 */
	@Override
	public Object write(DeviceConnection connection, List<ChannelValueContainer> containers,
			Object containerListHandle, int timeout) throws UnsupportedOperationException, ConnectionException {
		throw new UnsupportedOperationException();
	}

}
